import { QWidget } from './QWidget';
import { QFrame, QFrameSignals } from './QFrame';
import { ScrollBarPolicy } from '../QtEnums/ScrollBarPolicy';
import { QSize } from '../QtCore/QSize';
import { QScrollBar } from './QScrollBar';
/**

> Abstract class to add functionalities common to all scrollarea based widgets.

**This class implements all methods, properties of Qt's [QAbstractScrollArea class](https://doc.qt.io/qt-6/qabstractscrollarea.html) so that it can be inherited by all scroll based widgets**

`QAbstractScrollArea` is an abstract class and hence no instances of the same should be created. It exists so that we can add similar functionalities to all scrollable widget's easily. If you wish to create a scrollarea use QScrollArea instead.

**QAbstractScrollArea is the base class for all widgets.**

QAbstractScrollArea will list all methods and properties that are common to all scrollable widgets in the NodeGui world.

 */
export declare abstract class QAbstractScrollArea<Signals extends QAbstractScrollAreaSignals> extends QFrame<Signals> {
    horizontalScrollBar(): QScrollBar;
    maximumViewportSize(): QSize;
    setHorizontalScrollBar(scrollBar: QScrollBar): void;
    setHorizontalScrollBarPolicy(policy: ScrollBarPolicy): void;
    setViewport(widget: QWidget): void;
    setVerticalScrollBar(scrollBar: QScrollBar): void;
    setVerticalScrollBarPolicy(policy: ScrollBarPolicy): void;
    verticalScrollBar(): QScrollBar;
    viewport(): QWidget;
}
export type QAbstractScrollAreaSignals = QFrameSignals;
