#pragma once

#include <napi.h>

#include <QPointer>

#include "Extras/Export/export.h"
#include "QtCore/QObject/qobject_macro.h"
#include "napplication.hpp"

class DLL_EXPORT QApplicationWrap : public Napi::ObjectWrap<QApplicationWrap> {
  QOBJECT_WRAPPED_METHODS_DECLARATION

 private:
  QPointer<NApplication> instance;
  bool _wasManuallyCreated = false;

 public:
  static Napi::FunctionReference constructor;
  static Napi::Object init(Napi::Env env, Napi::Object exports);
  QApplicationWrap(const Napi::CallbackInfo& info);
  ~QApplicationWrap();
  NApplication* getInternalInstance();
  // Wrapped methods
  Napi::Value processEvents(const Napi::CallbackInfo& info);
  Napi::Value exec(const Napi::CallbackInfo& info);
  Napi::Value quit(const Napi::CallbackInfo& info);
  Napi::Value exit(const Napi::CallbackInfo& info);
  Napi::Value setQuitOnLastWindowClosed(const Napi::CallbackInfo& info);
  Napi::Value quitOnLastWindowClosed(const Napi::CallbackInfo& info);
  Napi::Value palette(const Napi::CallbackInfo& info);
  Napi::Value setStyleSheet(const Napi::CallbackInfo& info);
  Napi::Value devicePixelRatio(const Napi::CallbackInfo& info);
};

namespace StaticQApplicationWrapMethods {
DLL_EXPORT Napi::Value clipboard(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value instance(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value primaryScreen(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value screens(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value setStyle(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value style(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value setWindowIcon(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value windowIcon(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value applicationDisplayName(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value platformName(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value desktopFileName(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value desktopSettingsAware(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value setApplicationDisplayName(
    const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value setDesktopFileName(const Napi::CallbackInfo& info);
DLL_EXPORT Napi::Value setDesktopSettingsAware(const Napi::CallbackInfo& info);
}  // namespace StaticQApplicationWrapMethods
