#include "core/Events/eventsmap.h"

std::unordered_map<std::string, int> EventsMap::eventTypes{
    {"None", QEvent::None},
    {"ActionAdded", QEvent::ActionAdded},
    {"ActionChanged", QEvent::ActionChanged},
    {"ActionRemoved", QEvent::ActionRemoved},
    {"ActivationChange", QEvent::ActivationChange},
    {"ApplicationActivate", QEvent::ApplicationActivate},
    {"ApplicationActivated", QEvent::ApplicationActivated},
    {"ApplicationDeactivate", QEvent::ApplicationDeactivate},
    {"ApplicationFontChange", QEvent::ApplicationFontChange},
    {"ApplicationLayoutDirectionChange",
     QEvent::ApplicationLayoutDirectionChange},
    {"ApplicationPaletteChange", QEvent::ApplicationPaletteChange},
    {"ApplicationStateChange", QEvent::ApplicationStateChange},
    {"ApplicationWindowIconChange", QEvent::ApplicationWindowIconChange},
    {"ChildAdded", QEvent::ChildAdded},
    {"ChildPolished", QEvent::ChildPolished},
    {"ChildRemoved", QEvent::ChildRemoved},
    {"Clipboard", QEvent::Clipboard},
    {"Close", QEvent::Close},
    {"CloseSoftwareInputPanel", QEvent::CloseSoftwareInputPanel},
    {"ContentsRectChange", QEvent::ContentsRectChange},
    {"ContextMenu", QEvent::ContextMenu},
    {"CursorChange", QEvent::CursorChange},
    {"DeferredDelete", QEvent::DeferredDelete},
    {"DragEnter", QEvent::DragEnter},
    {"DragLeave", QEvent::DragLeave},
    {"DragMove", QEvent::DragMove},
    {"Drop", QEvent::Drop},
    {"DynamicPropertyChange", QEvent::DynamicPropertyChange},
    {"EnabledChange", QEvent::EnabledChange},
    {"Enter", QEvent::Enter},
    {"EnterWhatsThisMode", QEvent::EnterWhatsThisMode},
    {"Expose", QEvent::Expose},
    {"FileOpen", QEvent::FileOpen},
    {"FocusIn", QEvent::FocusIn},
    {"FocusOut", QEvent::FocusOut},
    {"FocusAboutToChange", QEvent::FocusAboutToChange},
    {"FontChange", QEvent::FontChange},
    {"Gesture", QEvent::Gesture},
    {"GestureOverride", QEvent::GestureOverride},
    {"GrabKeyboard", QEvent::GrabKeyboard},
    {"GrabMouse", QEvent::GrabMouse},
    {"GraphicsSceneContextMenu", QEvent::GraphicsSceneContextMenu},
    {"GraphicsSceneDragEnter", QEvent::GraphicsSceneDragEnter},
    {"GraphicsSceneDragLeave", QEvent::GraphicsSceneDragLeave},
    {"GraphicsSceneDragMove", QEvent::GraphicsSceneDragMove},
    {"GraphicsSceneDrop", QEvent::GraphicsSceneDrop},
    {"GraphicsSceneHelp", QEvent::GraphicsSceneHelp},
    {"GraphicsSceneHoverEnter", QEvent::GraphicsSceneHoverEnter},
    {"GraphicsSceneHoverLeave", QEvent::GraphicsSceneHoverLeave},
    {"GraphicsSceneHoverMove", QEvent::GraphicsSceneHoverMove},
    {"GraphicsSceneMouseDoubleClick", QEvent::GraphicsSceneMouseDoubleClick},
    {"GraphicsSceneMouseMove", QEvent::GraphicsSceneMouseMove},
    {"GraphicsSceneMousePress", QEvent::GraphicsSceneMousePress},
    {"GraphicsSceneMouseRelease", QEvent::GraphicsSceneMouseRelease},
    {"GraphicsSceneMove", QEvent::GraphicsSceneMove},
    {"GraphicsSceneResize", QEvent::GraphicsSceneResize},
    {"GraphicsSceneWheel", QEvent::GraphicsSceneWheel},
    {"Hide", QEvent::Hide},
    {"HideToParent", QEvent::HideToParent},
    {"HoverEnter", QEvent::HoverEnter},
    {"HoverLeave", QEvent::HoverLeave},
    {"HoverMove", QEvent::HoverMove},
    {"IconDrag", QEvent::IconDrag},
    {"IconTextChange", QEvent::IconTextChange},
    {"InputMethod", QEvent::InputMethod},
    {"InputMethodQuery", QEvent::InputMethodQuery},
    {"KeyboardLayoutChange", QEvent::KeyboardLayoutChange},
    {"KeyPress", QEvent::KeyPress},
    {"KeyRelease", QEvent::KeyRelease},
    {"LanguageChange", QEvent::LanguageChange},
    {"LayoutDirectionChange", QEvent::LayoutDirectionChange},
    {"LayoutRequest", QEvent::LayoutRequest},
    {"Leave", QEvent::Leave},
    {"LeaveWhatsThisMode", QEvent::LeaveWhatsThisMode},
    {"LocaleChange", QEvent::LocaleChange},
    {"NonClientAreaMouseButtonDblClick",
     QEvent::NonClientAreaMouseButtonDblClick},
    {"NonClientAreaMouseButtonPress", QEvent::NonClientAreaMouseButtonPress},
    {"NonClientAreaMouseButtonRelease",
     QEvent::NonClientAreaMouseButtonRelease},
    {"NonClientAreaMouseMove", QEvent::NonClientAreaMouseMove},
    {"MacSizeChange", QEvent::MacSizeChange},
    {"MetaCall", QEvent::MetaCall},
    {"ModifiedChange", QEvent::ModifiedChange},
    {"MouseButtonDblClick", QEvent::MouseButtonDblClick},
    {"MouseButtonPress", QEvent::MouseButtonPress},
    {"MouseButtonRelease", QEvent::MouseButtonRelease},
    {"MouseMove", QEvent::MouseMove},
    {"MouseTrackingChange", QEvent::MouseTrackingChange},
    {"Move", QEvent::Move},
    {"NativeGesture", QEvent::NativeGesture},
    {"OrientationChange", QEvent::OrientationChange},
    {"Paint", QEvent::Paint},
    {"PaletteChange", QEvent::PaletteChange},
    {"ParentAboutToChange", QEvent::ParentAboutToChange},
    {"ParentChange", QEvent::ParentChange},
    {"PlatformPanel", QEvent::PlatformPanel},
    {"PlatformSurface", QEvent::PlatformSurface},
    {"Polish", QEvent::Polish},
    {"PolishRequest", QEvent::PolishRequest},
    {"QueryWhatsThis", QEvent::QueryWhatsThis},
    {"ReadOnlyChange", QEvent::ReadOnlyChange},
    {"RequestSoftwareInputPanel", QEvent::RequestSoftwareInputPanel},
    {"Resize", QEvent::Resize},
    {"ScrollPrepare", QEvent::ScrollPrepare},
    {"Scroll", QEvent::Scroll},
    {"Shortcut", QEvent::Shortcut},
    {"ShortcutOverride", QEvent::ShortcutOverride},
    {"Show", QEvent::Show},
    {"ShowToParent", QEvent::ShowToParent},
    {"SockAct", QEvent::SockAct},
    {"StateMachineSignal", QEvent::StateMachineSignal},
    {"StateMachineWrapped", QEvent::StateMachineWrapped},
    {"StatusTip", QEvent::StatusTip},
    {"StyleChange", QEvent::StyleChange},
    {"TabletMove", QEvent::TabletMove},
    {"TabletPress", QEvent::TabletPress},
    {"TabletRelease", QEvent::TabletRelease},
    {"TabletEnterProximity", QEvent::TabletEnterProximity},
    {"TabletLeaveProximity", QEvent::TabletLeaveProximity},
    {"TabletTrackingChange", QEvent::TabletTrackingChange},
    {"ThreadChange", QEvent::ThreadChange},
    {"Timer", QEvent::Timer},
    {"ToolBarChange", QEvent::ToolBarChange},
    {"ToolTip", QEvent::ToolTip},
    {"ToolTipChange", QEvent::ToolTipChange},
    {"TouchBegin", QEvent::TouchBegin},
    {"TouchCancel", QEvent::TouchCancel},
    {"TouchEnd", QEvent::TouchEnd},
    {"TouchUpdate", QEvent::TouchUpdate},
    {"UngrabKeyboard", QEvent::UngrabKeyboard},
    {"UngrabMouse", QEvent::UngrabMouse},
    {"UpdateLater", QEvent::UpdateLater},
    {"UpdateRequest", QEvent::UpdateRequest},
    {"WhatsThis", QEvent::WhatsThis},
    {"WhatsThisClicked", QEvent::WhatsThisClicked},
    {"Wheel", QEvent::Wheel},
    {"WinEventAct", QEvent::WinEventAct},
    {"WindowActivate", QEvent::WindowActivate},
    {"WindowBlocked", QEvent::WindowBlocked},
    {"WindowDeactivate", QEvent::WindowDeactivate},
    {"WindowIconChange", QEvent::WindowIconChange},
    {"WindowStateChange", QEvent::WindowStateChange},
    {"WindowTitleChange", QEvent::WindowTitleChange},
    {"WindowUnblocked", QEvent::WindowUnblocked},
    {"WinIdChange", QEvent::WinIdChange},
    {"ZOrderChange", QEvent::ZOrderChange},
};
